using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace AccelerometerGraph
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        const int ticksPerSecond = 30;
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        int displayWidth, displayHeight;
        Texture2D backgroundTexture;
        Texture2D graphTexture;
        uint[] pixels;
        int totalTicks;
        int previousInsertRow;
        Vector3 previousAcceleration;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Zune.
            TargetElapsedTime = TimeSpan.FromSeconds(1 / 30.0);
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            displayWidth = this.GraphicsDevice.Viewport.Width;
            displayHeight = this.GraphicsDevice.Viewport.Height;

            // Create background texture and initialize it
            backgroundTexture = new Texture2D(this.GraphicsDevice, displayWidth, ticksPerSecond);
            pixels = new uint[backgroundTexture.Width * backgroundTexture.Height];

            for (int y = 0; y < backgroundTexture.Height; y++)
                for (int x = 0; x < backgroundTexture.Width; x++)
                {
                    Color clr = Color.Black;

                    if (y == 0 || x == backgroundTexture.Width / 2 ||
                                  x == backgroundTexture.Width / 4 ||
                                  x == 3 * backgroundTexture.Width / 4)
                    {
                        clr = new Color(128, 128, 128);
                    }
                    else if (y % 5 == 0 || ((x - backgroundTexture.Width / 2) % (backgroundTexture.Width / 16) == 0))
                    {
                        clr = new Color(64, 64, 64);
                    }

                    pixels[y * backgroundTexture.Width + x] = clr.PackedValue;
                }
            backgroundTexture.SetData<uint>(pixels);

            // Create graph texture
            graphTexture = new Texture2D(this.GraphicsDevice, displayWidth, displayHeight);
            pixels = new uint[graphTexture.Width * graphTexture.Height];

            // Initialize
            previousInsertRow = graphTexture.Height - 2;
        }

        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            Vector3 acceleration = Accelerometer.GetState().Acceleration;

            totalTicks = (int)Math.Round(gameTime.TotalGameTime.TotalSeconds / this.TargetElapsedTime.TotalSeconds); 
            int insertRow = (totalTicks + graphTexture.Height - 1) % graphTexture.Height;

            // newInsertRow is always guaranteed to be greater than previousInsertRow,
            //  but might also be greater than the graphTexture height!
            int newInsertRow = insertRow < previousInsertRow ? insertRow + graphTexture.Height : insertRow;

            for (int y = previousInsertRow + 1; y <= newInsertRow; y++)
                for (int x = 0; x < graphTexture.Width; x++)
                    pixels[(y % graphTexture.Height) * graphTexture.Width + x] = 0;

            DrawLines(graphTexture, pixels, previousInsertRow, newInsertRow, previousAcceleration, acceleration);
            graphTexture.SetData<uint>(pixels);

            previousInsertRow = insertRow;
            previousAcceleration = acceleration;

            base.Update(gameTime);
        }

        void DrawLines(Texture2D texture, uint[] pixels, int prevRow, int newRow, Vector3 prevAcc, Vector3 newAcc)
        {
            DrawLine(texture, pixels, prevRow, newRow, prevAcc.X, newAcc.X, Color.Red);
            DrawLine(texture, pixels, prevRow, newRow, prevAcc.Y, newAcc.Y, Color.Green);
            DrawLine(texture, pixels, prevRow, newRow, prevAcc.Z, newAcc.Z, Color.Blue);
        }

        void DrawLine(Texture2D texture, uint[] pixels, int prevRow, int newRow, float prevAcc, float newAcc, Color clr)
        {
            DrawLine(texture, pixels, texture.Width / 2 + (int)(prevAcc * texture.Width / 4), prevRow,
                                      texture.Width / 2 + (int)(newAcc * texture.Width / 4), newRow, clr); 
        }

        void DrawLine(Texture2D texture, uint[] pixels, int x1, int y1, int x2, int y2, Color clr)
        {
            if (x1 == x2 && y1 == y2)
            {
                SetPixel(texture, pixels, x1, y1, clr); 
            }

            else if (Math.Abs(y2 - y1) > Math.Abs(x2 - x1))
            {
                int sign = Math.Sign(y2 - y1);

                for (int y = y1; y != y2; y += sign)
                {
                    float fraction = (float)(y - y1) / (y2 - y1);
                    int x = (int)(x1 + fraction * (x2 - x1)); 
                    SetPixel(texture, pixels, x, y, clr);
                }
            }
            else
            {
                int sign = Math.Sign(x2 - x1);

                for (int x = x1; x != x2; x += sign)
                {
                    float fraction = (float)(x - x1) / (x2 - x1);
                    int y = (int)(y1 + fraction * (y2 - y1)); 
                    SetPixel(texture, pixels, x, y, clr);
                }
            }
        }

        // Note use of bitwise OR!
        void SetPixel(Texture2D texture, uint[] pixels, int x, int y, Color clr)
        {
            pixels[(y % texture.Height) * texture.Width + x] |= clr.PackedValue;
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            // Draw background texture
            int displayRow = -totalTicks % backgroundTexture.Height;

            while (displayRow < displayHeight)
            {
                spriteBatch.Draw(backgroundTexture, new Vector2(0, displayRow), Color.White);
                displayRow += backgroundTexture.Height;
            }

            // Draw graph texture
            displayRow = -totalTicks % graphTexture.Height;

            while (displayRow < displayHeight)
            {
                spriteBatch.Draw(graphTexture, new Vector2(0, displayRow), Color.White);
                displayRow += graphTexture.Height;
            }
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
 